<?php
include_once(dirname(__FILE__)."/common.php");
class KeywordCheckPrepareAction extends SOYShopSitePrepareAction{

	private $config;

	/**
	 * @return string
	 */
	function prepare(){
		
		//カートページでは動作しない
		$cartUri = soyshop_get_cart_uri();
		if(preg_match('/'.$cartUri.'$/',$_SERVER["REQUEST_URI"]) > 0 || strpos($_SERVER["REQUEST_URI"],"/".$cartUri."/")!==false)return;
		
		//マイページでは動作しない
		$mypageUri = soyshop_get_mypage_uri();
		if(preg_match('/'.$mypageUri.'$/',$_SERVER["REQUEST_URI"]) > 0 || strpos($_SERVER["REQUEST_URI"],"/".$mypageUri."/")!==false)return;
			
		$config = UtilKeywordCheckCommon::getConfig();
				
		//URLにすでにプリフィックスがついている場合は処理を行わない→次に違うワードでアクセスした場合、セッションの値はfalseになるはず
		if(preg_match('/'.$config["prefix"].'$/',$_SERVER["REQUEST_URI"]) > 0 || strpos($_SERVER["REQUEST_URI"],"/".$config["prefix"]."/")!==false)return;
		
		//既にチェックが終了していて、リダイレクトを行う必要がある時
		$isRedirect = (isset($config["redirect"]) && $config["redirect"] == 1) ? true : false;
		if($isRedirect===true){
			$checkKeyword = $this->getAttribute();
			if($checkKeyword===true){
				//リダイレクト先の絶対パス
				$path = $this->getRedirectPath($config["prefix"]);
				SOY2PageController::redirect($path);
			}
		}
		
		//そもそも他ページから来ていない場合は実行しない
		$referer = (isset($_SERVER["HTTP_REFERER"])) ? $_SERVER["HTTP_REFERER"] : null;
//		$referer = $this->debug("google");

		//リファラ内のURLのドメインと取得したHTTP_HOSTの値が同じ場合は処理をやめる
		if(strpos($referer,$_SERVER["HTTP_HOST"]))return;
		
		//リファラが取れなかった時、またはリファラにGETの値がなかった場合は処理をやめる
		if(is_null($referer) && strpos($referer,"?")===false)return;
				
		$keywords = (isset($config["keywords"]) && strlen($config["keywords"]) > 0) ? $config["keywords"] : null;
		
		//キーワードが無い時は処理を終了する
		if(is_null($keywords))return;
		
		//キーワードを一旦配列にばらす
		$list = explode(",",$keywords);
		
		$keywords = "";	//一旦文字列を空にする
		
		//Googleから遷移してきた場合は、リファラモード
		if(preg_match('/^http:\/\/www.google/',$referer)){
			
			//getにoqがある場合は、oqを読みに行く
			if(preg_match('/oq=/',$referer)){
				preg_match('/&oq=(.*?)&/',$referer,$result);
			//getにqのみの場合
			}else{
				preg_match('/&q=(.*?)&/',$referer,$result);
			}
				
			$keywordString = $this->getKeyword($result[1]);
			
		//Yahooから遷移してきた場合は、リファラモード
		}elseif(preg_match('/^http:\/\/search.yahoo/',$referer)){
			
			//一番初めの検索ワードを取得
			preg_match('/p=(.*?)&/',$referer,$result);
			$keywordString = $this->getKeyword($result[1]);
		
		//Bingから遷移してきた場合は、リファラモード
		}elseif(preg_match('/^http:\/\/www.bing/',$referer)){
			
			//一番初めの検索ワードを取得
			preg_match('/q=(.*?)&/',$referer,$result);
			$keywordString = $this->getKeyword($result[1]);
		
		//それ以外の場合は今のところ、何もしない
		}else{
			$this->setAttribute(false);
			return;
		}
				
		$keywords = explode(",",$keywordString);	//検索キーワード
		
		//検索されたキーワードに設定したキーワードが入っているかどうかのフラグ
		$hitKeyword = false;
		
		//ここで配列を比較して、一つでもマッチしたらtrueを返す
		$compares = array_intersect($list,$keywords);
		if(count($compares) > 0){
			$hitKeyword = true;	
		}
				
		//trueの場合はその旨をセッションに放り込み、リダイレクト
		if($hitKeyword===true){
			$this->setAttribute(true);

			//リダイレクトの設定を行なっている時はリダイレクトするz
			if($isRedirect===true){
				//リダイレクト先の絶対パス
				$path = $this->getRedirectPath($config["prefix"]);
				SOY2PageController::redirect($path);
			}
		}else{
			//ヒットしていなくても、一応セッションに値を入れておく
			$this->setAttribute(false);			
		}
	}
	
	/**
	 * URLにプレフィックスを付けた絶対パスを返す
	 */
	function getRedirectPath($prefix){
		//REQUEST_URI
		$requestUri = rawurldecode($_SERVER['REQUEST_URI']);
		//getの値を取り除く
		if(strpos($requestUri,"?")!==false){
			$requestUri = substr($requestUri,0,strpos($requestUri,"?"));
		}

		//PATH_INFO
		$pathInfo = isset($_SERVER['PATH_INFO']) ? $_SERVER['PATH_INFO'] : "" ;
		//先頭はスラッシュ
		if(strlen($pathInfo) && $pathInfo[0]!=="/"){
			$pathInfo = "/".$pathInfo;
		}

		//無限ループになるときはfalseを返す
		if( $pathInfo === "/".$prefix || strpos($pathInfo,"/".$prefix."/")===0 ){
			return false;
		}

		//サイトID：最初と最後に/を付けておく
		$siteDir = strlen($pathInfo) ? strtr($requestUri,array($pathInfo => "")) : $requestUri ;//strtrのキーは空文字列であってはいけない
		if($siteDir[0]!== "/"){
			$siteDir = "/".$siteDir;
		}
		if(substr($siteDir, -1) !== "/"){
			$siteDir = $siteDir. "/";
		}

		//prefixを付ける
		$path = $siteDir. $prefix. $pathInfo;

		//絶対パスにQuery Stringを追加する
		if(isset($_SERVER["QUERY_STRING"]) && strlen($_SERVER["QUERY_STRING"]) > 0){
			$path = rtrim($path,"/");
			if(strpos($_SERVER["QUERY_STRING"],session_name()."=")!==false){
				$querystring = preg_replace("/".session_name()."=[A-Za-z0-9]*/",session_name()."=".session_id(),$_SERVER["QUERY_STRING"]);
			}else{
				$querystring = $_SERVER["QUERY_STRING"];
			}

			$path .= "?" . $querystring;
		}

		return $path;
	}
	
	/**
	 * 検索用のqueryとログ用の検索ワードを取得する
	 * @param str
	 * @return keyword
	 */
	function getKeyword($str){
		
		if(strpos($str,"+")){
			$keyword = str_replace("+",",",$str);
		}elseif(strpos($str,"%20")){
			$keyword = str_replace("%20",",",$str);
		}elseif(strpos($str,"　")){
			$keyword = str_replace("　",",",$str);
		}
		
		return $keyword;
	}
	
	function getAttribute(){
		return UtilKeywordCheckCommon::getKeywordToSession();
	}
	
	function setAttribute($value){
		UtilKeywordCheckCommon::setKeywordToSession($value);
	}
	
	function debug($site){
		$list = $this->debugList();
		$url = $list[$site];
		return $url;
	}
	
	function debugList(){
		return array(
			"google" => "http://www.google.co.jp/url?sa=t&source=web&cd=2&ved=0CCgQFjAB&url=http%3A%2F%2Fwww.soyshop.net%2F&rct=j&q=soy%20lpo&ei=mig1TsOVAcvUmAWPk_3wCg&usg=AFQjCNFboas6kOqNXS3YebTjW39OhiZ7sg&sig2=HzYQL9ZSDNIt044iF6mtIw",
			"yahoo" => "http://search.yahoo.co.jp/search?p=soy+lpo&search.x=1&fr=top_ga1_sa&tid=top_ga1_sa&ei=UTF-8&aq=&oq=",
			"bing" => "http://www.bing.com/search?q=soy+lpo&form=ASUMHP&qs=n&sk=&pc=ASU2&x=0&y=0",
			"regoogle" => "http://www.google.co.jp/search?q=soy&ie=utf-8&oe=utf-8&aq=t&rls=org.mozilla:ja:official&hl=ja&client=firefox-a#sclient=psy&hl=ja&client=firefox-a&hs=AE3&rls=org.mozilla:ja%3Aofficial&source=hp&q=soy+shop&pbx=1&oq=soy+shop&aq=f&aqi=g4&aql=&gs_sm=e&gs_upl=23851l23851l3l24049l1l1l1l0l1l0l0l0ll0l0&bav=on.2,or.r_gc.r_pw.&fp=290ed995c4ecddcd&biw=1163&bih=590"
		);
	}
}

SOYShopPlugin::extension("soyshop.site.prepare","util_keyword_check","KeywordCheckPrepareAction");